
from ctypes import *
from ctypes.wintypes import *
import os
import wx
import addonHandler
import gui
import watchdog
from logHandler import log
import config
from . import licenseExtension
addonHandler.initTranslation()
try:
    libUSA = windll.libUSA
    raise LookupError
except:
    currentPath = os.path.abspath(os.path.dirname(__file__))
    addonCodePath = currentPath.rsplit('\\', 2)[0]
    libUSAPath = os.path.join(addonCodePath, "visionEnhancementProviders\\_base\\libUSA.dll")
    log.info(libUSAPath)
    libUSA = windll.LoadLibrary(libUSAPath)


ActivateLicenseFuncType = WINFUNCTYPE(c_wchar_p, c_wchar_p, POINTER(BOOL))
DeactivateLicenseFuncType = WINFUNCTYPE(c_wchar_p, POINTER(BOOL))
IsLicensedFuncType = WINFUNCTYPE(BOOL)
_ActivateLicense=ActivateLicenseFuncType(('USA_ActivateLicense', libUSA),((1,'licenseKey'),(1,'success')))
_DeactivateLicense = DeactivateLicenseFuncType(('USA_DeactivateLicense', libUSA),((1,'success'),))
IsLicensed = IsLicensedFuncType(("USA_IsLicensed", libUSA))

def ActivateLicense(licenseKey):
    success = BOOL()
    message = _ActivateLicense(licenseKey, byref(success))
    return (success, message)

def DeactivateLicense():
    success = BOOL()
    message = _DeactivateLicense(byref(success))
    return (success, message)



class USALicenseDialog(wx.Dialog):

    def __init__(self, parent):
        # Translators: Title of the license dialog
        super().__init__(parent, id=wx.ID_ANY, title=_("Manage USA License"))
        self.isLicensed = IsLicensed()
        addonManifest = addonHandler.getCodeAddon().manifest
        currentVersion = addonManifest['version']
        if self.isLicensed:
            message = _("Your copy of Ultimate Screen Access {currentVersion} is properly licensed.\nChoose OK to deactivate this specific license of this product or Cancel to close this dialog.").format(currentVersion=currentVersion)
        else:
            message = _("Your copy of Ultimate Screen Access {currentVersion} doesn't seem to be licensed.\nEnter your license key in the respective field and choose OK to activate the product. \nAlternatively, press Cancel to close this dialog or deactivate all licenses of USA on this computer.").format(currentVersion=currentVersion)
        main_sizer = wx.BoxSizer(wx.VERTICAL)
        main_sizer.Add(wx.StaticText(self, label=message))
        main_sizer_helper = gui.guiHelper.BoxSizerHelper(self, orientation=wx.VERTICAL)
        #Translators: The input field to enter the USA license key
        self.key = main_sizer_helper.addLabeledControl(_("&License Key:"), wx.TextCtrl)
        self.key.Enabled=not self.isLicensed
        offlineActivation = main_sizer_helper.addItem(wx.Button(self, label=_("Offline activation")))
        offlineActivation.Bind(wx.EVT_BUTTON, self.offlineActivation)
        autoActivation = main_sizer_helper.addItem(wx.Button(self, label=_("(BETA) Automatic activation")))
        autoActivation.Bind(wx.EVT_BUTTON, self.automaticActivation)
        main_sizer_helper.addDialogDismissButtons(self.CreateButtonSizer(wx.OK | wx.CANCEL))

        self.Bind(wx.EVT_BUTTON, self.on_ok, id=wx.ID_OK)
        main_sizer.Add(main_sizer_helper.sizer, border = gui.guiHelper.BORDER_FOR_DIALOGS, flag=wx.ALL)
        main_sizer.Fit(self)
        self.SetSizer(main_sizer)
        self.Center(wx.BOTH )
        self.Show()
        self.key.SetFocus()

    def on_ok(self, evt):
        if not self.isLicensed:
            self.activate(evt)
        else:
            self.deactivate(evt)

    def activate(self, evt):
        if not self.key.Value:
            gui.messageBox(_("You must enter a valid license key."), _("Error"), wx.OK | wx.ICON_ERROR)
            self.key.SetFocus()
            return
        progressDialog = gui.IndeterminateProgressDialog(self, _("Performing request"), _("Please wait while your license is being activated..."))

        try:

            success, message = watchdog.cancellableExecute(ActivateLicense, self.key.Value)
        except:
            success = False
            message = _("There was an error while performing your request.")
            log.error("Activation error", exc_info=True)

        if not success:
            wx.CallAfter(gui.messageBox,
                _("An error has occured:\n{error}").format(error=message),
                    _("Error"), wx.OK | wx.ICON_ERROR)
        else:
            evt.Skip()
            wx.CallAfter(		gui.messageBox,
                _("Ultimate Screen Access has been activated!\nAdditional info: {message}").format(message=message),
                    _("Congratulations!"), wx.OK | wx.ICON_EXCLAMATION)
        progressDialog.done()

    def deactivate(self, evt):
        progressDialog = gui.IndeterminateProgressDialog(self, _("Performing request"), _("Please wait while your license is being deactivated..."))

        try:

            success, message = watchdog.cancellableExecute(DeactivateLicense)
        except:
            success = False
            message = _("There was a timeout while performing your request.")
            log.error("Activation error", exc_info=True)

        if not success:
            wx.CallAfter(gui.messageBox,
                _("An error has occured:\n{error}").format(error=message),
                    _("Error"), wx.OK | wx.ICON_ERROR)
        else:
            evt.Skip()
            wx.CallAfter(gui.messageBox,
                _("Ultimate Screen Access has been deactivated!\nAdditional info: {message}").format(message=message),
                    _("Congratulations!"), wx.OK | wx.ICON_EXCLAMATION)
        progressDialog.done()

    def offlineActivation(self, evt):

        with offlineDialog(self) as dlg:
            dlg.ShowModal()

        self.Close()
        self.Destroy()

    def automaticActivation(self, evt):

        with automaticActivationDialog(self) as dlg:
            dlg.ShowModal()

        self.Close()
        self.Destroy()

class automaticActivationDialog(wx.Dialog):

    def __init__(self, parent):
        # Translators: Title of the license dialog
        super().__init__(parent, id=wx.ID_ANY, title=_("automatic Activation"))
        self.isLicensed = IsLicensed()
        addonManifest = addonHandler.getCodeAddon().manifest
        currentVersion = addonManifest['version']

        self.autoActivationEnabled = config.conf.get("usaAutoActivation", False)
        self.licenseForAutoActivation = config.conf.get("usaAutoActivationLicenseKey", "")

        message = _("We offer automatic activations for non-presistent enviroments. With this USA communicates with the license server whenever it starts up.")

        main_sizer = wx.BoxSizer(wx.VERTICAL)
        main_sizer.Add(wx.StaticText(self, label=message))
        main_sizer_helper = gui.guiHelper.BoxSizerHelper(self, orientation=wx.VERTICAL)

        productChoices = ["USA 2022", "USA 2023", "USA 2024", "USA 2025"]
        self.productIDs = [1073, 1355, 1356, 1357]
        self.regPath = ["USA2022", "USA2023", "USA2024", "USA2025"]
        self.productID = main_sizer_helper.addLabeledControl("&license version", wx.Choice, choices=productChoices)
        self.productID.SetSelection(0)

        self.lic = main_sizer_helper.addLabeledControl(_("&license key:"), wx.TextCtrl)
        self.lic.Enabled = True
        self.lic.Value = self.licenseForAutoActivation

        self.automaticActivation = main_sizer_helper.addItem(wx.Button(self, label=_("enable automatic activation")))
        self.automaticActivation.Bind(wx.EVT_BUTTON, self.automaticActivation_Click)

        if self.autoActivationEnabled:
            self.automaticActivation.SetLabel(_("disable automatic activation"))
        else:
            self.automaticActivation.SetLabel(_("enable automatic activation"))

        main_sizer_helper.addDialogDismissButtons(self.CreateButtonSizer(wx.OK | wx.CANCEL))
        self.Bind(wx.EVT_BUTTON, self.on_ok, id=wx.ID_OK)
        main_sizer.Add(main_sizer_helper.sizer, border = gui.guiHelper.BORDER_FOR_DIALOGS, flag=wx.ALL)
        main_sizer.Fit(self)
        self.SetSizer(main_sizer)
        self.Center(wx.BOTH )
        self.Show()

    def automaticActivation_Click(self, evt):

        if not self.autoActivationEnabled and not len(self.lic.Value.strip()) > 0:
            gui.messageBox("fill in a license key!")
            return


        self.autoActivationEnabled = not self.autoActivationEnabled
        if self.autoActivationEnabled:
            self.automaticActivation.SetLabel(_("disable automatic activation"))
            productID = self.productIDs[self.productID.GetSelection()]
            license = self.lic.Value
            config.conf['usaAutoActivationProductID'] = productID
            config.conf['usaAutoActivationLicenseKey'] = license
            gui.messageBox("automatic activation enabled")



        else:
            self.automaticActivation.SetLabel(_("enable automatic activation"))
            gui.messageBox("automatic activation disabled")

        evt.Skip()

    def on_ok(self, evt):
        config.conf['usaAutoActivation'] = self.autoActivationEnabled
        self.Close()
        self.Destroy()

class offlineDialog(wx.Dialog):

    def __init__(self, parent):
        # Translators: Title of the license dialog
        super().__init__(parent, id=wx.ID_ANY, title=_("offline Activation"))
        self.isLicensed = IsLicensed()
        addonManifest = addonHandler.getCodeAddon().manifest
        currentVersion = addonManifest['version']

        message = _("We offer offline activations as well. For this you need to mail your licensekey and instance ID to support@babbage.com. We will send you an unique activation hash so that USA is licensed without the need of verifying online.")

        main_sizer = wx.BoxSizer(wx.VERTICAL)
        main_sizer.Add(wx.StaticText(self, label=message))
        main_sizer_helper = gui.guiHelper.BoxSizerHelper(self, orientation=wx.VERTICAL)
        #Translators: The input field to enter the USA license key


        productChoices = ["USA 2022", "USA 2023", "USA 2024", "USA 2025"]
        self.productIDs = [1073, 1355, 1356, 1357]
        self.regPath = ["USA2022", "USA2023", "USA2024", "USA2025"]
        self.productID = main_sizer_helper.addLabeledControl("&license version", wx.Choice, choices=productChoices)
        self.productID.SetSelection(0)
        self.productID.Bind(wx.EVT_CHOICE, self.changedProduct)

        id = licenseExtension.getInstanceID(self.productIDs[self.productID.GetSelection()])

        self.key = main_sizer_helper.addLabeledControl(_("&Instance ID:"), wx.TextCtrl)
        self.key.Enabled = True
        self.key.Value = id
        self.lic = main_sizer_helper.addLabeledControl(_("&license key:"), wx.TextCtrl)
        self.lic.Enabled = True
        self.activationHash = main_sizer_helper.addLabeledControl(_("&Activation hash:"), wx.TextCtrl)
        self.activationHash.Enabled = True

        main_sizer_helper.addDialogDismissButtons(self.CreateButtonSizer(wx.OK | wx.CANCEL))
        self.Bind(wx.EVT_BUTTON, self.on_ok, id=wx.ID_OK)
        main_sizer.Add(main_sizer_helper.sizer, border = gui.guiHelper.BORDER_FOR_DIALOGS, flag=wx.ALL)
        main_sizer.Fit(self)
        self.SetSizer(main_sizer)
        self.Center(wx.BOTH )
        self.Show()
        self.key.SetFocus()


    def changedProduct(self, evt):
        id = licenseExtension.getInstanceID(self.productIDs[self.productID.GetSelection()])
        self.key.Value = id
        evt.Skip()
    def on_ok(self, evt):
        if len(self.activationHash.Value.split()) > 0:

            regPath = os.path.join("SOFTWARE", "ACCESS4U",self.regPath[self.productID.GetSelection()])

            log.info("writing to registry")
            licenseExtension.saveDataToRegistry(self.lic.Value, self.activationHash.Value, regPath)


        self.Close()
        self.Destroy()



