# This code is based on work by Joseph Lee


import globalVars
if globalVars.appArgs.secure:
	raise RuntimeError("NVDA in secure mode, cannot check for add-on update")
try:
	import updateCheck
except RuntimeError:
	# Ugly hack
	import versionInfo
	versionInfo.updateVersionType = "USA"
	import updateCheck
import config
import addonHandler
import os
import threading
from urllib.request import urlopen
import tempfile
import ctypes
import ssl
import hashlib
import time
import re
import config
import gui
import wx
import urllib.request
import json
from logHandler import log
from . import licensing
addonHandler.initTranslation()

addonUpdateCheckInterval = 86400

channels={
	"latest": "https://www.babbage.com/Gebruikers/Falko/allProducts.json",
}

updateChecker = None
# To avoid freezes, a background thread will run after the global plugin constructor calls wx.CallAfter.
def autoUpdateCheck():
	currentTime = time.time()
	whenToCheck = config.conf["vision"]["usa"]["updateCheckTime"] + (config.conf["vision"]["usa"]["updateCheckTimeInterval"] * addonUpdateCheckInterval)
	if currentTime >= whenToCheck:
		t = threading.Thread(target=addonUpdateCheck, kwargs={"autoCheck":True})
		t.daemon = True
		t.start()
	else: startAutoUpdateCheck(interval=divmod((whenToCheck-currentTime), addonUpdateCheckInterval)[-1])

# Start or restart auto update checker.
def startAutoUpdateCheck(interval=None):
	# Don't do anything if told to check for update whenever NVDA starts.
	if not config.conf["vision"]["usa"]["updateCheckTimeInterval"]: return
	global updateChecker
	if updateChecker is not None:
		wx.CallAfter(updateChecker.Stop)
	updateChecker = wx.PyTimer(autoUpdateCheck)
	wx.CallAfter(updateChecker.Start, (addonUpdateCheckInterval if interval is None else interval) * 1000, True)

# Borrowed ideas from NVDA Core.


def checkLicenseForOnlineProducts():
	updateURL = channels["latest"]
	with urllib.request.urlopen(updateURL) as url:
		data = json.loads(url.read().decode())

	for ii in data['versions']:
		lm = licensing.LicenseManager(  ii['id'], 'USA')
		if lm.obtainValidLicense():
			log.info("Valid license for USA obtained for product ID" + str(ii['id']) )
			return ii['link']

	return None

def checkForAddonUpdate():
	addonManifest = addonHandler.getCodeAddon().manifest

	updateURL = checkLicenseForOnlineProducts()
	if updateURL:
		try:
			res = urlopen(updateURL)
			res.close()
		except IOError as e:
			# SSL issue (seen in NVDA Core earlier than 2014.1).
			if isinstance(e.strerror, ssl.SSLError) and e.strerror.reason == "CERTIFICATE_VERIFY_FAILED":
				_updateWindowsRootCertificates()
				res = urlopen(updateURL)
			else:
				raise
		if res.code != 200:
			raise RuntimeError("Checking for update failed with code %d" % res.code)
		# Build emulated add-on update dictionary if there is indeed a new version.
		NewVersion = re.search("USA-(?P<version>.*).nvda-addon", res.url).groupdict()["version"]
		currentVersion = addonManifest['version']

		if shouldUpdate(currentVersion, NewVersion):
			return {"curVersion": currentVersion, "newVersion": NewVersion, "path": res.url}
	return None

def shouldUpdate(current, other):
	# here I assume that version numbers are just numbers like 2020.4.1
	intCurrent = [int(ii) for ii in current.split('.')]
	intOther = [int(ii) for ii in other.split('.')]
	n1 = len(intCurrent)
	n2 = len(intOther)
	n = min(n1, n2)

	for ii in range(0, n):
		if intCurrent[ii] < intOther[ii]:
			return True
		elif intCurrent[ii] > intOther[ii]:
			return False
	return n1 < n2



progressDialog = None
def addonUpdateCheck(autoCheck=False):
	global progressDialog
	config.conf["vision"]["usa"]["updateCheckTime"] = int(time.time())
	try:
		info = checkForAddonUpdate()
	except:
		log.error("Error checking for update", exc_info=True)
		if not autoCheck:
			wx.CallAfter(progressDialog.done)
			progressDialog = None
			# Translators: Error text shown when add-on update check fails.
			wx.CallAfter(gui.messageBox, _("Error checking for update."),
				# Translators: Title of the dialog shown when add-on update check fails.
				_("Ultimate Screen Access update"), wx.ICON_ERROR)
		else: startAutoUpdateCheck()
		return
	if not autoCheck:
		wx.CallAfter(progressDialog.done)
		progressDialog = None
	if info is None:
		if not autoCheck:
			# Translators: Presented when no add-on update is available.
			wx.CallAfter(gui.messageBox, _("No add-on update available."),
				# Translators: Title of the dialog presented when no add-on update is available.
				_("Ultimate Screen Access update"))
			return
		else: startAutoUpdateCheck()
	else:
		if autoCheck: startAutoUpdateCheck()
		# Translators: Text shown if an add-on update is available.
		checkMessage = _("Ultimate Screen Access {newVersion} is available. Would you like to update?").format(newVersion = info["newVersion"])
		# Translators: Title of the add-on update check dialog.
		wx.CallAfter(getUpdateResponse, checkMessage, _("Ultimate Screen Access update"), info["path"])

def getUpdateResponse(message, caption, updateURL):
	if gui.messageBox(message, caption, wx.YES_NO | wx.NO_DEFAULT | wx.CENTER | wx.ICON_QUESTION) == wx.YES:
		USAUpdateDownloader([updateURL]).start()

# Update downloader (credit: NV Access)
# Customized for USA add-on.
class USAUpdateDownloader(updateCheck.UpdateDownloader):
	"""Overrides NVDA Core's downloader.)
	No hash checking for now, and URL's and temp file paths are different.
	"""

	def __init__(self, urls, fileHash=None):
		"""Constructor.
		@param urls: URLs to try for the update file.
		@type urls: list of str
		@param fileHash: The SHA-1 hash of the file as a hex string.
		@type fileHash: basestring
		"""
		self.urls = urls
		self.destPath = tempfile.mktemp(prefix="USA_update-", suffix=".nvda-addon")
		self.fileHash = fileHash

	def start(self):
		"""Start the download.
		"""
		self._shouldCancel = False
		# Use a timer because timers aren't re-entrant.
		self._guiExecTimer = wx.PyTimer(self._guiExecNotify)
		gui.mainFrame.prePopup()
		# Translators: The title of the dialog displayed while downloading add-on update.
		self._progressDialog = wx.ProgressDialog(_("Downloading Add-on Update"),
			# Translators: The progress message indicating that a connection is being established.
			_("Connecting"),
			# PD_AUTO_HIDE is required because ProgressDialog.Update blocks at 100%
			# and waits for the user to press the Close button.
			style=wx.PD_CAN_ABORT | wx.PD_ELAPSED_TIME | wx.PD_REMAINING_TIME | wx.PD_AUTO_HIDE,
			parent=gui.mainFrame)
		self._progressDialog.Raise()
		t = threading.Thread(target=self._bg)
		t.daemon = True
		t.start()

	def _error(self):
		self._stopped()
		gui.messageBox(
			# Translators: A message indicating that an error occurred while downloading an update to NVDA.
			_("Error downloading add-on update."),
			# Message included in NVDA Core
			_("Error"),
			wx.OK | wx.ICON_ERROR)

	def _download(self, url):
		remote = urlopen(url, timeout=120)
		if remote.code != 200:
			raise RuntimeError("Download failed with code %d" % remote.code)
		size = int(remote.headers["content-length"])
		local = open(self.destPath, "wb")
		if self.fileHash:
			hasher = hashlib.sha1()
		self._guiExec(self._downloadReport, 0, size)
		read = 0
		chunk=8192
		while True:
			if self._shouldCancel:
				return
			if size -read <chunk:
				chunk =size -read
			block = remote.read(chunk)
			if not block:
				break
			read += len(block)
			if self._shouldCancel:
				return
			local.write(block)
			if self.fileHash:
				hasher.update(block)
			self._guiExec(self._downloadReport, read, size)
		if read < size:
			raise RuntimeError("Content too short")
		if self.fileHash and hasher.hexdigest() != self.fileHash:
			raise RuntimeError("Content has incorrect file hash")
		local.close()
		self._guiExec(self._downloadReport, read, size)

	def _downloadSuccess(self):
		self._stopped()
		gui.mainFrame.prePopup()
		# Emulate add-on update (don't prompt to install).
		from gui import addonGui
		if addonGui.installAddon(gui.mainFrame, self.destPath):
			addonGui.promptUserForRestart()
		else:
			# Translators: The message displayed when an error occurs when installing an add-on package.
			gui.messageBox(_("Failed to update add-on  from %s")%self.destPath,
				_("Error"),
				wx.OK | wx.ICON_ERROR)
		gui.mainFrame.postPopup()
		try:
			os.remove(self.destPath)
		except OSError:
			pass

# Borrowed from NVDA Core (the only difference is the URL and where structures are coming from).
def _updateWindowsRootCertificates():
	crypt = ctypes.windll.crypt32
	# Get the server certificate.
	sslCont = ssl._create_unverified_context()
	u = urlopen("https://access4u.eu", context=sslCont)
	cert = u.fp._sock.getpeercert(True)
	u.close()
	# Convert to a form usable by Windows.
	certCont = crypt.CertCreateCertificateContext(
		0x00000001, # X509_ASN_ENCODING
		cert,
		len(cert))
	# Ask Windows to build a certificate chain, thus triggering a root certificate update.
	chainCont = ctypes.c_void_p()
	crypt.CertGetCertificateChain(None, certCont, None, None,
		ctypes.byref(updateCheck.CERT_CHAIN_PARA(cbSize=ctypes.sizeof(updateCheck.CERT_CHAIN_PARA),
			RequestedUsage=updateCheck.CERT_USAGE_MATCH())),
		0, None,
		ctypes.byref(chainCont))
	crypt.CertFreeCertificateChain(chainCont)
	crypt.CertFreeCertificateContext(certCont)
