"""
Extra lib for wordfollower
"""
import NVDAObjects.UIA.wordDocument
from speech.commands import CallbackCommand
import textInfos
import re
import ctypes
from comtypes import COMError
import locationHelper
from logHandler import log
from NVDAObjects.window import winword
from NVDAObjects.UIA import UIATextInfo
from compoundDocuments import CompoundTextInfo
import virtualBuffers
import textInfos.offsets

import time
import api
import winUser
import mouseHandler

re_white_space = re.compile(r"(\s+|$)", re.DOTALL)

sayAllWordDataBase ={}
jj = 0

def moveStartEndOffsetsByMove(info, bookmark):
    # NVDA knows different textInfos objects that have different implementations for moving the bookmark location.
    # this functions should generalize this by using the always available move function

    # the bookmark that is returned can be of the class Offsets or others, like wordDocumentTextInfo, here the right
    # info is extracted
    if isinstance(bookmark, textInfos.offsets.Offsets):
        start = bookmark.startOffset
        end = bookmark.endOffset
    elif isinstance(bookmark, UIATextInfo) or isinstance(bookmark, CompoundTextInfo):
        #log.info(bookmark)
        #raise NotImplementedError
        info = bookmark
        return info

    else:
        log.info(bookmark)
        raise NotImplementedError

    # we can always get the text by move, but it is very slowly for some vbs. so we implement a simple offset based workaround
    if isinstance(info, textInfos.offsets.OffsetsTextInfo):
        info._startOffset = start
        info._endOffset = end
    else:
        info.move(textInfos.UNIT_STORY, direction=-1, endPoint='start')
        info.move(textInfos.UNIT_STORY, direction=-1, endPoint='end')
        info.move(textInfos.UNIT_CHARACTER, end, endPoint="end")
        info.move(textInfos.UNIT_CHARACTER, start, endPoint="start")
    return info

def getTextBookmark(text, whiteSpaceTracker):
    # returns the bookmark of the text. whiteSpaceTracker should already have the startpoint at the text.
    whiteSpaceTracker.move(textInfos.UNIT_CHARACTER, len(text), endPoint="end")
    bookmark = whiteSpaceTracker._get_bookmark()
    return bookmark

def makeCallBackWrapper(ii, callBack):

    def _callBackWrapper(qq = ii):
        return callBack(qq)

    return _callBackWrapper


def wordSplitter( info, sequenceOld, callBack, offset = 0):
    # This function splits the speech sequence (sequenceOld) into separate words with their respective callback command.

    # copy incoming sequence. If it fails we can return the original
    sequence = sequenceOld.copy()

    # copy textInfo of sayall Reader to split words at correct offset
    whiteSpaceTracker = info.copy()
    whiteSpaceTracker.collapse()
    whiteSpaceTracker.move(textInfos.UNIT_CHARACTER, offset)

    # for vbufs the text can dynamically change, so throwing back a bookmark can result in a wrong position. textTracker
    # is used to track the beginning of the text and possible changes to it.
    textTracker = whiteSpaceTracker.copy()
    textTracker.move(textInfos.UNIT_STORY, direction=-1, endPoint='start')
    textTracker.move(textInfos.UNIT_STORY, direction=1, endPoint='end')
    originalText = textTracker.text


    # forloop over all commands in sequence
    for index, command in list(enumerate(sequence)):
        # skip if it is not a string
        if not isinstance(command, str):
            continue

        # make sub-sequence to order callback commands and text
        curCommandSequence = []
        # indices of white spaces
        endOfWhiteSpaceIndexes = [m.end() for m in re_white_space.finditer(command)]

        # word counter and offset start
        start = 0
        ii = 0
        # split up string in words
        for end in endOfWhiteSpaceIndexes:
            text = command[start:end]
            if not text == '':

                # checks ends of the whitespacetracker and that of the textInfo that is read from
                if whiteSpaceTracker.compareEndPoints(info, "startToEnd") > 0:
                    log.info(f"break cause compareEndPoints {whiteSpaceTracker.compareEndPoints(info, 'startToEnd') } {text} ")
                    break


                # get the bookmark location of the word and put it in the callback function so we can jump to it when it is spoken
                bookmark = getTextBookmark(text, whiteSpaceTracker)

                # in the case of UIA, it returns a UIATextInfo again. We want to send a copy of it to the callback command
                if isinstance(bookmark, UIATextInfo): bookmark = bookmark.copy()

                # put all data in the sayAll global Dict for storage
                global jj, sayAllWordDataBase
                jj += 1
                sayAllWordDataBase[jj] = [text, bookmark, originalText, textTracker]

                _callBackWrapper = makeCallBackWrapper(jj, callBack)

                # move the whiteSpaceTracker to the next word
                whiteSpaceTracker.move(textInfos.UNIT_CHARACTER, len(text), 'start')

                #append callback command and the text to the subsequence
                curCommandSequence.append( CallbackCommand(_callBackWrapper, name=text) )
                curCommandSequence.append(text)

                # increment word counter
                ii += 1
            # The whiteSpaceTracker shouldn't move past the end of the info we're speaking.
            start = end

        # add subsequence in the correct location
        sequence[index] = curCommandSequence

    # the list "sequence" now has some sublists in it. We need to make this in just one contiuous list, this is what
    # the following code does
    expandedRelativeSpeechSequence = []
    for x in sequence:
        if isinstance(x, list):
            expandedRelativeSpeechSequence.extend(x)
        else:
            expandedRelativeSpeechSequence.append(x)

    if not expandedRelativeSpeechSequence:
        # the other functions do count on the fact that something is returned. by splitting text into words this is not gauranteed.
        # in that case we return the original sequence, since it probably is a empty string
        return sequenceOld
    return expandedRelativeSpeechSequence


def getBoundingRects(updater):
    rect = None
    try:
        if isinstance(updater, NVDAObjects.UIA.wordDocument.WordDocumentTextInfo):
            #updater._ensureRangeVisibility()
            # copied from _esnureRangeVisiblity(), added forced scroll when topLeft is visible but rightBottom isnt
            try:
                inView = updater.pointAtStart in updater.obj.location and updater.boundingRects[0].bottomRight in updater.obj.location
            except LookupError:
                inView = False
            if not inView:
                    updater._rangeObj.ScrollIntoView(True)

        elif isinstance(updater, textInfos.TextInfo ):
            updater.NVDAObjectAtStart.scrollIntoView()

        # try getting the rect the proper way
        rect = updater._get_boundingRects()
    except Exception as e:
        # maybe we are in word
        if isinstance(updater, winword.WordDocumentTextInfo):
            rect = Word_get_boundingRects(updater)
            return rect
        else:
            log.error(f"SPEECH: {updater} could not use _get_boundingRects()")

    # if we got a rect convert it to RectLTWH
    if rect:
        rect = rect[0]
        rect = locationHelper.RectLTWH(left=rect.left, top=rect.top, width=rect.width, height=rect.height)
    else:
        # otherwise try gettin the rect an alternative way. This works for most objects just outside the monitor view
        try:
            left = updater._getBoundingRectFromOffset(updater._startOffset).left
            top = updater._getBoundingRectFromOffset(updater._startOffset).top
            right = updater._getBoundingRectFromOffset(updater._endOffset).right
            bottom = updater._getBoundingRectFromOffset(updater._endOffset).bottom
            rect = locationHelper.RectLTWH(left=left, top=top, width=right-left, height=bottom-top)
        except:
            rect = None
            return rect

    #rect = scrollIntoView(rect, updater)
    return rect


def Word_get_boundingRects(wordTextInfo):
    left = ctypes.c_int()
    top = ctypes.c_int()
    width = ctypes.c_int()
    height = ctypes.c_int()
    try:

        wordTextInfo.obj.WinwordWindowObject.GetPoint(ctypes.byref(left), ctypes.byref(top), ctypes.byref(width),
                                                      ctypes.byref(height), wordTextInfo._rangeObj)
    except COMError:
        raise LookupError
    if not any((left.value, top.value, width.value, height.value)):
        raise LookupError
    return locationHelper.RectLTWH(left=left.value, top=top.value, height=height.value, width=width.value)


